#include "GLImage.h"

// base constructor, Create a new image from a file name
graphic::GLImage::GLImage(std::string file)
{
	// Temporary surface
	SDL_Surface * temp;

	// Load image
	temp = IMG_Load(file.c_str());

	// An error occurred
	if(temp == NULL)
		throw Exception("GLImage", IMG_GetError());

	if(temp->format->BitsPerPixel == ALPHA_BPP)
	{
		_surface = SDL_DisplayFormatAlpha(temp);
		_uses_alpha = true;
	}
	else
	{
		_surface = SDL_DisplayFormat(temp);
		_uses_alpha = false;
	}

	SDL_SetAlpha(_surface, SDL_RLEACCEL, SDL_ALPHA_OPAQUE);

	// Free temporary surface
	SDL_FreeSurface(temp);

	// An error occurred
	if(_surface == NULL)
		throw Exception("GLImage", IMG_GetError());

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting image dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;

	// no trans color
	_has_trans_color = false;
	_trans_color = 0;

	_need_tex_update = false;

	// Checking for max texture size
	if (!CheckTexSize(_surface))
		throw Exception("GLSurface", "Max texture size exceeded\n");

	// Setting real power-of-two texture dimensions
	_wreal = (int)powf(2.0, ceilf(logf((float)_surface->w)/logf(2.0f)));
	_hreal = (int)powf(2.0, ceilf(logf((float)_surface->h)/logf(2.0f)));

	// Texture creation
	if (!Surf2Tex(_surface))
		throw Exception("GLImage", "Texture not created, image format unknown\n");

	SetVisibleLimits();
}

// create a new image setting a color as transparent
graphic::GLImage::GLImage(std::string file, SDL_Color trans_color)
{
	// Temporary surface
	SDL_Surface *temp;

	// Load image
	temp = IMG_Load(file.c_str());

	// An error occurred
	if(temp == NULL)
		throw Exception("GLImage", IMG_GetError());

	_surface = SDL_DisplayFormat(temp);

	SDL_SetAlpha(_surface, SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
	
	// Free temporary surface
	SDL_FreeSurface(temp);

	// an error occurred
	if(_surface == NULL)
		throw Exception("GLImage", IMG_GetError());

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting image dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;

	// set trans color
	_has_trans_color = true;
	_trans_color = SDL_MapRGB(_surface->format, trans_color.r, trans_color.g, trans_color.b);
	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY|SDL_RLEACCEL, _trans_color);
	
	// Set alpha flag
	_uses_alpha = false;

	_need_tex_update = false;

	// Checking for max texture size
	if (!CheckTexSize(_surface))
		throw Exception("GLSurface", "Max texture size exceeded\n");

	// Setting real power-of-two texture dimensions
	_wreal = (int)powf(2.0, ceilf(logf((float)_surface->w)/logf(2.0f)));
	_hreal = (int)powf(2.0, ceilf(logf((float)_surface->h)/logf(2.0f)));

	// Texture creation
	if (!Surf2Tex(_surface))
		throw Exception("GLImage", "Texture not created, image format unknown\n");

	SetVisibleLimits();
}

// return true if the pixel at (x,y) is transparent, false otherwise
bool graphic::GLImage::IsTrans(Uint16 x, Uint16 y)
{
	Lock();

	if((_uses_alpha && GetAlpha(x, y) <= MIN_TRANS) ||
	   (!_uses_alpha && _surface->format->alpha <= MIN_TRANS) ||
	   (_has_trans_color && GetPixel(x, y) == _trans_color))
		return true;
	else
		return false;

	Unlock();
}

// set visible limits of the image
void graphic::GLImage::SetVisibleLimits()
{
	int first_x = _surface->w - 1;
	int first_y = _surface->h - 1;
	int last_x = 0;
	int last_y = 0;

	// Set visible area - the image uses alpha or has a trans color
	if(_uses_alpha || _has_trans_color || _opacity < SDL_ALPHA_OPAQUE)
	{
		Lock();

		for(int x = 0; x < _surface->w; x++)
		{
			for(int y = 0; y < _surface->h; y++)
			{
				// Opaque color
				if((_uses_alpha && GetAlpha(x, y) > MIN_TRANS) ||
				   (!_uses_alpha && !_has_trans_color &&  _surface->format->alpha > MIN_TRANS) ||
				   (_has_trans_color && GetPixel(x, y) != _trans_color && _surface->format->alpha > MIN_TRANS))
				{
					if(x < first_x)
						first_x = x;
					if(x > last_x)
						last_x = x;

					if(y < first_y)
						first_y = y;
					if(y > last_y)
						last_y = y;
				}
			}
		}

		Unlock();

		_vis_rect->x = first_x;
		_vis_rect->y = first_y;
		_vis_rect->w = last_x;
		_vis_rect->h = last_y;
	}
	// Set visible area - the image doesn't use alpha nor have a trans color ==> all the image
	else
	{
		_vis_rect->x = 0;
		_vis_rect->y = 0;
		_vis_rect->w = _rectangle->w - 1;
		_vis_rect->h = _rectangle->h - 1;
	}
}
