#include "GLSurface.h"

// create a surface starting from a SDL_Surface
graphic::GLSurface::GLSurface(SDL_Surface * surface)
{
	// Set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	_need_tex_update = false;

	// Store the SDL_Surface
	_surface = surface;

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;

	// Checking for max texture size
	if (!CheckTexSize(_surface))
		throw Exception("GLSurface", "Max texture size exceeded\n");
	
	// Setting real power-of-two texture dimensions
	_wreal = (int)powf(2.0, ceilf(logf((float)surface->w)/logf(2.0f)));
	_hreal = (int)powf(2.0, ceilf(logf((float)surface->h)/logf(2.0f)));

	SDL_SetAlpha(surface, SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
	
	if (!Surf2Tex(_surface))
		throw Exception("GLSurface", "Texture not created, image format unknown\n");
}

// a new, not inizialized, surface
graphic::GLSurface::GLSurface(Uint16 w, Uint16 h, Uint32 flags, bool trans, Uint16 bpp)
{
	Uint32 rmask, gmask, bmask, amask;

	// transparent surface
	if(trans)
	{
		#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
		#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
		#endif
	}
	// opaque surface
	else
	{
		rmask = 0;
		gmask = 0;
		bmask = 0;
		amask = 0;
	}

	// Set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	_need_tex_update = false;

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = w;
	_rectangle->h = h;

	// create surface
	_surface = SDL_CreateRGBSurface(flags, w, h, bpp, rmask, gmask, bmask, amask);

	SDL_SetAlpha(_surface, SDL_RLEACCEL, SDL_ALPHA_OPAQUE);

	// an error has occurred
	if(_surface == NULL)
		throw Exception("GLSurface", SDL_GetError());

	// Checking for max texture size
	if (!CheckTexSize(_surface))
		throw Exception("GLSurface", "Max texture size exceeded\n");
	
	// Setting real texture dimensions to be power-of-two
	_wreal = (int)powf(2.0, ceilf(logf((float)_surface->w)/logf(2.0f)));
	_hreal = (int)powf(2.0, ceilf(logf((float)_surface->h)/logf(2.0f)));

	if (!Surf2Tex(_surface))
		throw Exception("GLSurface", "Texture not created, image format unknown\n");
}

// A new, not inizialized, surface
graphic::GLSurface::GLSurface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags)
{
	// Set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	_need_tex_update = false;

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = w;
	_rectangle->h = h;

	// create surface
	_surface = SDL_CreateRGBSurface(flags|SDL_SRCALPHA,w,h, format->BitsPerPixel,
									format->Rmask, format->Gmask, format->Bmask, format->Amask );

	// an error has occurred
	if(_surface == NULL)
		throw Exception("GLSurface", SDL_GetError());

	// Checking for max texture size
	if (!CheckTexSize(_surface))
		throw Exception("GLSurface", "Max texture size exceeded\n");

	// Setting real power-of-two texture dimensions	
	_wreal = (int)powf(2.0, ceilf(logf((float)_surface->w)/logf(2.0f)));
	_hreal = (int)powf(2.0, ceilf(logf((float)_surface->h)/logf(2.0f)));

	if (!Surf2Tex(_surface))
		throw Exception("GLSurface", "Texture not created, image format unknown\n");

	SDL_SetAlpha(_surface, SDL_RLEACCEL, SDL_ALPHA_OPAQUE);
}

// check for max texture size exceeding
bool graphic::GLSurface::CheckTexSize(SDL_Surface * surface)
{
	GLint max_size;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_size);
	if ((surface->w > max_size) || (surface->h > max_size))
		return false;
	else
		return true;
}

// make a texture from the class surface
bool graphic::GLSurface::Surf2Tex(SDL_Surface * surface)
{
	SDL_Surface *image;
	Uint32 rmask, gmask, bmask, amask;
	int xpad, ypad;
	
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	rmask = 0xff000000; gmask = 0x00ff0000; bmask = 0x0000ff00;	amask = 0x000000ff;
	#else
	rmask = 0x000000ff;	gmask = 0x0000ff00;	bmask = 0x00ff0000;	amask = 0xff000000;
	#endif

	image = SDL_CreateRGBSurface(SDL_SWSURFACE, _wreal, _hreal, 32, rmask, gmask, bmask, amask);

	// NOT POT
	if(_wreal != surface->w || _hreal != surface->h)
	{
		xpad = (_wreal - surface->w)/2;
		ypad = (_hreal - surface->h)/2;
	}
	// POT
	else
	{
		xpad = ypad = 0;
	}

	SDL_Rect dst = {xpad, ypad, surface->w, surface->h};
	SDL_BlitSurface(surface, NULL, image, &dst);

	glGenTextures(1, &_tex_num);
	glBindTexture(GL_TEXTURE_2D, _tex_num);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image->w, image->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

	// Free temporary surface
	SDL_FreeSurface(image);

	return true;
}

// texture update from backup surface
void graphic::GLSurface::UpdateTex(bool del_old_tex)
{
	if(del_old_tex)
		glDeleteTextures(1, &_tex_num);

	if (!Surf2Tex(_surface))
		throw Exception("GLSurface", "Texture not created, image format unknown\n");

	// no more updates required
	_need_tex_update = false;
}

// fill part of a surface with a color
void graphic::GLSurface::Fill(SDL_Color & color, SDL_Rect & rect)
{
	// use a temporary rectangle to avoid data loss
	SDL_Rect frect;
	frect.x = rect.x;
	frect.y = rect.y;
	frect.w = rect.w;
	frect.h = rect.h;

	// fill
	SDL_FillRect(_surface, &frect, SDL_MapRGB(_surface->format, color.r, color.g, color.b));

	_need_tex_update = true;
}

// get color of a pixel
Uint32 graphic::GLSurface::GetPixel(Sint16 x, Sint16 y)
{
	int bpp = _surface->format->BytesPerPixel;

	// p is the address of the pixel we want to retrieve
	Uint8 * p = (Uint8 *)_surface->pixels + y * _surface->pitch + x * bpp;

	switch(bpp)
	{
		case 1:
			return *p;

		case 2:
			return *(Uint16 *)p;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case 4:
			return *(Uint32 *)p;

		default:
			return 0;
	}
}

// set opacity of the surface
void graphic::GLSurface::SetOpacity(Uint8 opacity)
{
	// Save the opacity value
	_opacity = opacity;
}

// set data of the pixel at (x,y) - NEED Lock/Unlock
void graphic::GLSurface::SetPixel(Sint16 x, Sint16 y, Uint32 data)
{
	int bpp = _surface->format->BytesPerPixel;

	/* Here p is the address of the pixel we want to set */
	Uint8 *p = (Uint8 *)_surface->pixels + y * _surface->pitch + x * bpp;

	switch(bpp)
	{
		case 1:
			*p = data;
			break;

		case 2:
			*(Uint16 *)p = data;
			break;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			{
				p[0] = (data >> 16) & 0xff;
				p[1] = (data >> 8) & 0xff;
				p[2] = data & 0xff;
			}
			else
			{
				p[0] = data & 0xff;
				p[1] = (data >> 8) & 0xff;
				p[2] = (data >> 16) & 0xff;
			}
			break;

		case 4:
			*(Uint32 *)p = data;
			break;

		default:
			break;
	}
}

