#ifndef CLASS_GL_SURFACE
#define CLASS_GL_SURFACE

#include <GL/gl.h>
#include <math.h>

#include "Screen.h"
#include "Surface.h"

namespace graphic
{

/// Basic GL surface, just a rectangle that you can blit or fill with a color
class GLSurface : public virtual Surface
{
	protected:
	 /// The surface used to read pixels values from
	 SDL_Surface * _surface;
	 /// Texture identificator
	 GLuint _tex_num;
	 /// Real texture dimension (in case padding is required)
	 int _wreal, _hreal;
	 /// this flag is true if the texture needs an update
	 bool _need_tex_update;

	 /// Check for max texture size exceeding
	 bool CheckTexSize(SDL_Surface * surface);
	 /// Make a texture from a surface
	 bool Surf2Tex(SDL_Surface * surface);

	public:
	 /// Base constructor
	 GLSurface() 
	 {
	 	_opacity =  SDL_ALPHA_OPAQUE;
		_tex_num = 0;
		_need_tex_update = false;
	 };
	 /// Create a surface starting from a SDL_Surface
	 GLSurface(SDL_Surface * surface);
	 /// a new, not inizialized, surface
	 GLSurface(Uint16 w, Uint16 h, Uint32 flags = SDL_SWSURFACE, bool trans = false,
				Uint16 bpp = screen->GetBitPP());
	 /// a new, not inizialized, surface
	 GLSurface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags = SDL_SWSURFACE);
	 /// Surface data is freed
	 virtual ~GLSurface()
	 {
	 	glDeleteTextures(1, &_tex_num);
	 	SDL_FreeSurface(_surface);
	 };

	 /// Return texture id for the blitter to blit
	 GLuint GetTexNum(void) { return _tex_num; };
	 /// Return the real texture width
	 int GetWReal(void) { return _wreal; };
	 /// Return the real texture height
	 int GetHReal(void) { return _hreal; };

	 /// Set the opacity of the surface
	 virtual void SetOpacity(Uint8 opacity);

	 /// Set surface color
	 void Fill(SDL_Color & color)
	 {
		 SDL_FillRect(_surface, NULL, SDL_MapRGB(_surface->format, color.r, color.g, color.b));
		_need_tex_update = true;
	 };

	 /// Fill part of the surface with a color
	 void Fill(SDL_Color & color, SDL_Rect & f_rect);

	 /// Texture update from backup surface
	 void UpdateTex(bool del_old_tex = true);

	 /// Get color of a pixel
	 Uint32 GetPixel(Sint16 x, Sint16 y);
	 /// Set data of the pixel at (x,y)
	 void SetPixel(Sint16 x, Sint16 y, Uint32 data);

	 /// Set R,G,B,A values of a pixel at (x,y) - NEED Lock/Unlock
	 void GetRGBA(Sint16 x, Sint16 y, Uint8 * r, Uint8 * g, Uint8 * b, Uint8 * a)
	 {
		SDL_GetRGBA(GetPixel(x, y), _surface->format, r, g, b, a);
	 };

	 /// Return Alpha value of a pixel at (x,y) - NEED Lock/Unlock
	 Uint8 GetAlpha(Sint16 x, Sint16 y)
	 {
	 	Uint8 r, g, b, a;
	 	SDL_GetRGBA(GetPixel(x, y), _surface->format, &r, &g, &b, &a);
	 	return a;
	 };

	 /// Lock the surface so to permit direct pixel access under SDL
	 void Lock()
	 {
		 if(SDL_MUSTLOCK(_surface))
			 SDL_LockSurface(_surface);
	 };
	 /// Release the previous lock
	 void Unlock()
	 {
		 if(SDL_MUSTLOCK(_surface))
			 SDL_UnlockSurface(_surface);
	 };

	 /// return true if the texture needs an update
	 bool NeedTexUpdate() { return _need_tex_update; };
	 /// set tex update flag to true
	 void RequestTexUpdate() { _need_tex_update = true; };

	 /// GLBlitter is a friend of GLSurface in order to access to its data
  	 friend class GLBlitter;
};

}

#endif
