#include "SDLImage.h"

// base constructor, Create a new image from a file name, by default no alpha
graphic::SDLImage::SDLImage(std::string file)
{
	// Temporary surface
	SDL_Surface * temp;

	// Load image
	temp = IMG_Load(file.c_str());

	// an error is occurred
	if(temp == NULL)
		throw Exception("SDLImage", IMG_GetError());

	if(temp->format->BitsPerPixel == ALPHA_BPP)
	{
		_surface = SDL_DisplayFormatAlpha(temp);
		_uses_alpha = true;
	}
	else
	{
		_surface = SDL_DisplayFormat(temp);
		_uses_alpha = false;
	}

	// Free temporary surface
	SDL_FreeSurface(temp);

	// an error is occurred
	if(_surface == NULL)
		throw Exception("SDLImage", IMG_GetError());

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting image dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;

	// no trans color
	_has_trans_color = false;
	_trans_color = 0;

	SetVisibleLimits();
}

// create a new image setting a color as transparent
graphic::SDLImage::SDLImage(std::string file, SDL_Color trans_color)
{
	// Temporary surface
	SDL_Surface * temp;
	Uint32 colorkey;

	// Load image
	temp = IMG_Load(file.c_str());

	// an error is occurred
	if(temp == NULL)
		throw Exception("SDLImage", IMG_GetError());

	_surface = SDL_DisplayFormat(temp);

	// Free temporary surface
	SDL_FreeSurface(temp);

	// an error is occurred
	if(_surface == NULL)
		throw Exception("SDLImage", IMG_GetError());

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting image dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;

	// Set colorkey as transparent
	colorkey = SDL_MapRGB(_surface->format, trans_color.r, trans_color.g, trans_color.b);
	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY|SDL_RLEACCEL, colorkey);

	// set trans color
	_has_trans_color = true;
	_trans_color = SDL_MapRGB(_surface->format, trans_color.r, trans_color.g, trans_color.b);

	 // set alpha flag
	_uses_alpha = false;

	SetVisibleLimits();
}

// return true if the pixel at (x,y) is transparent, false otherwise
bool graphic::SDLImage::IsTrans(Uint16 x, Uint16 y)
{
	Lock();

	if((_uses_alpha && GetAlpha(x, y) <= MIN_TRANS) ||
	   (!_uses_alpha && _surface->format->alpha <= MIN_TRANS) ||
	   (_has_trans_color && GetPixel(x, y) == _trans_color))
		return true;
	else
		return false;

	Unlock();
}

// set visible limits of the image
void graphic::SDLImage::SetVisibleLimits()
{
	int first_x = _surface->w - 1;
	int first_y = _surface->h - 1;
	int last_x = 0;
	int last_y = 0;

	// set visible area - the image uses alpha or has a trans color
	if(_uses_alpha || _has_trans_color || _opacity < SDL_ALPHA_OPAQUE)
	{
		Lock();

		for(int x = 0; x < _surface->w; x++)
		{
			for(int y = 0; y < _surface->h; y++)
			{
				// opaque color

				if((_uses_alpha && GetAlpha(x, y) > MIN_TRANS) ||
				   (!_uses_alpha && !_has_trans_color &&  _surface->format->alpha > MIN_TRANS) ||
				   (_has_trans_color && GetPixel(x, y) != _trans_color && _surface->format->alpha > MIN_TRANS))
				{
					if(x < first_x)
						first_x = x;
					if(x > last_x)
						last_x = x;

					if(y < first_y)
						first_y = y;
					if(y > last_y)
						last_y = y;
				}
			}
		}

		Unlock();

		_vis_rect->x = first_x;
		_vis_rect->y = first_y;
		_vis_rect->w = last_x;
		_vis_rect->h = last_y;
	}
	// set visible area - the image doesn't use alpha nor have a trans color ==> all the image
	else
	{
		_vis_rect->x = 0;
		_vis_rect->y = 0;
		_vis_rect->w = _rectangle->w - 1;
		_vis_rect->h = _rectangle->h - 1;
	}
}

