#include "SDLSurface.h"

// a new, not inizialized, surface
graphic::SDLSurface::SDLSurface(Uint16 w, Uint16 h, Uint32 flags, bool trans, Uint16 bpp)
{
	Uint32 rmask, gmask, bmask, amask;

	// transparent surface
	if(trans)
	{
		#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000; 
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
		#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
		#endif
	}
	// opaque surface
	else
	{
		rmask = 0;
		gmask = 0;
		bmask = 0;
		amask = 0;
	}

	// set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = w;
	_rectangle->h = h;

	// create surface
	_surface = SDL_CreateRGBSurface(flags, w, h, bpp, rmask, gmask, bmask, amask);

	SDL_SetAlpha(_surface, SDL_SRCALPHA|SDL_RLEACCEL , _opacity);

	// an error is occurred
	if(_surface == NULL)
		throw Exception("SDLSurface", SDL_GetError());

};

// create a surface starting from a SDL_Surface
graphic::SDLSurface::SDLSurface(SDL_Surface * surface)
{
	// set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	// store the SDL_Surface
	_surface = surface;

	SDL_SetAlpha(_surface, SDL_SRCALPHA|SDL_RLEACCEL , _opacity);

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = _surface->w;
	_rectangle->h = _surface->h;
}

// A new, not inizialized, surface
graphic::SDLSurface::SDLSurface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags)
{
	// set opacity to opaque (full visible)
	_opacity =  SDL_ALPHA_OPAQUE;

	// Setting default position
	_rectangle->x = 0;
	_rectangle->y = 0;
	// Setting surface dimensions
	_rectangle->w = w;
	_rectangle->h = h;

	// create surface
	_surface = SDL_CreateRGBSurface(flags, w, h, format->BitsPerPixel,
															format->Rmask, format->Gmask, format->Bmask, format->Amask );

	SDL_SetAlpha(_surface, SDL_SRCALPHA|SDL_RLEACCEL , _opacity);

	// an error is occurred
	if(_surface == NULL)
		throw Exception("SDLSurface", SDL_GetError());

};

// set opacity of the surface
void graphic::SDLSurface::SetOpacity(Uint8 opacity)
{
	Uint8 old_opacity = _opacity;
	_opacity = opacity;

	// RGB surface
	if(_surface->format->Amask == 0)
	{
		// set the opacity
		SDL_SetAlpha(_surface, SDL_SRCALPHA|SDL_RLEACCEL , opacity);

		return;
	}

	// RGBA surface

	Uint8 r, g, b, a;
	// ratio needed when old_opacity < opacity
	int ratio = opacity / old_opacity;
	// rest needed when old_opacity < opacity
	int rest = opacity % old_opacity;

	if(SDL_MUSTLOCK(_surface))
		SDL_LockSurface(_surface);

	for(int x = 0; x < _surface->w; x++)
	{
		for(int y = 0; y < _surface->h; y++)
		{
			// get alpha value of the pixel
			SDL_GetRGBA(GetPixel(x, y), _surface->format, &r, &g, &b, &a);

			// reduce the opacity
			if(old_opacity > opacity)
				a = (a * opacity) / old_opacity;
			// increase the opacity
			else if(old_opacity < opacity)
			{

				a =  a * ratio;
				// check if a + rest is bigger than the max value
				if((a + rest) > SDL_ALPHA_OPAQUE)
					a = SDL_ALPHA_OPAQUE;
				// add rest just if a is not really small (!= 0)
				else if(a > rest && rest <= MAX_REST)
					a += rest;
			}

			SetPixel(x, y, SDL_MapRGBA(_surface->format, r, g, b, a));
		}
	}

	if(SDL_MUSTLOCK(_surface))
		SDL_UnlockSurface(_surface);
}

// fill part of a surface with a color
void graphic::SDLSurface::Fill(SDL_Color & color, SDL_Rect & rect)
{
	// use a temporary rectangle to avoid data loss
	SDL_Rect frect;
	frect.x = rect.x;
	frect.y = rect.y;
	frect.w = rect.w;
	frect.h = rect.h;

	// fill
	SDL_FillRect(_surface, &frect, SDL_MapRGB(_surface->format, color.r, color.g, color.b));
}

// get data from the pixel at (x,y) - NEED Lock/Unlock
Uint32 graphic::SDLSurface::GetPixel(Sint16 x, Sint16 y)
{
	int bpp = _surface->format->BytesPerPixel;

	// p is the address to the pixel we want to retrieve
	Uint8 * p = (Uint8 *)_surface->pixels + y * _surface->pitch + x * bpp;

	switch(bpp)
	{
		case 1:
			return *p;

		case 2:
			return *(Uint16 *)p;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case 4:
			return *(Uint32 *)p;

		default:
			return 0;
	}
}

// set data of the pixel at (x,y) - NEED Lock/Unlock
void graphic::SDLSurface::SetPixel(Sint16 x, Sint16 y, Uint32 data)
{
	int bpp = _surface->format->BytesPerPixel;

	/* Here p is the address to the pixel we want to set */
	Uint8 *p = (Uint8 *)_surface->pixels + y * _surface->pitch + x * bpp;

	switch(bpp)
	{
		case 1:
			*p = data;
			break;

		case 2:
			*(Uint16 *)p = data;
			break;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			{
				p[0] = (data >> 16) & 0xff;
				p[1] = (data >> 8) & 0xff;
				p[2] = data & 0xff;
			}
			else
			{
				p[0] = data & 0xff;
				p[1] = (data >> 8) & 0xff;
				p[2] = (data >> 16) & 0xff;
			}
			break;

		case 4:
			*(Uint32 *)p = data;
			break;

		default:
			break;
	}
}

// return Alpha value of a pixel at (x,y) - NEED Lock/Unlock
Uint8 graphic::SDLSurface::GetAlpha(Sint16 x, Sint16 y)
{
	Uint8  a;
	Uint32 temp, pixel;
	SDL_PixelFormat * fmt;

	// get pixel
	pixel = GetPixel(x, y);

	// SDL pixel format structure
	fmt = _surface->format;

	// get Alpha component of the pixel
	temp = pixel & fmt->Amask;
	temp = temp >> fmt->Ashift;
	temp = temp << fmt->Aloss;
	a = (Uint8)temp;

	return a;
};


