#ifndef CLASS_SURFACE
#define CLASS_SURFACE

#include <SDL.h>

#include "../DataTypes.h"
#include "../Exception.h"

#define ALPHA_BPP 32
#define NO_ALPHA_BPP 24

namespace graphic
{

/// This is a minimal Surface, the super class/interface of concrete ones (SDL, OpenGL)
class Surface
{
	protected:
	 /// rectangle that represent the surface (x,y - w,h)
	 SDL_Rect * _rectangle;

	 /// opacity value of the surface, the value range is defined between
	 /// SDL_ALPHA_TRANSPARENT and SDL_ALPHA_OPAQUE
	 Uint8 _opacity;

	public:
	 /// base constructor
	 Surface() {  _rectangle = new SDL_Rect; };
	 /// destroyer
	 virtual ~Surface() { delete _rectangle; };

	 /// set position of the upper left Point of the surface (x,y)
	 /// passing a Point
	 void SetPosition(Point & position)
	 {
		 _rectangle->x = position.x;
		 _rectangle->y = position.y;
	 };
	 /// set position of the upper left Point of the surface (x,y)
	 /// passing x and y values
	 void SetPosition(Sint16 x, Sint16 y)
	 {
		 _rectangle->x = x;
		 _rectangle->y = y;
	 };

	 /// set X value of the upper left point of the surface
	 void SetX(Sint16 x) { _rectangle->x = x; };
	 /// set Y value of the upper left point of the surface
	 void SetY(Sint16 y) { _rectangle->y = y; };

	 /// get Surface X position (top left corner)
	 Sint16 GetX() { return _rectangle->x; };
	 /// get Surface Y position (top left corner)
	 Sint16 GetY() { return _rectangle->y; };
	 /// get Surface width
	 Uint16 GetW() { return _rectangle->w; };
	 /// get Surface height
	 Uint16 GetH() { return _rectangle->h; };

	 /// fill the surface with a color
	 virtual void Fill(SDL_Color & color) = 0;
	 /// fill part of the surface with a color
	 virtual void Fill(SDL_Color & color, SDL_Rect & rect) = 0;

	 /// set the opacity of the surface
	 virtual void SetOpacity(Uint8 opacity) = 0;
	 /// get the opacity value
	 Uint8 GetOpacity() { return _opacity; };

	 /// get data from the pixel at (x,y)
	 virtual Uint32 GetPixel(Sint16 x, Sint16 y) = 0;
	 /// set data of the pixel at (x,y)
	 virtual void SetPixel(Sint16 x, Sint16 y, Uint32 data) = 0;

	 /// set R,G,B,A values of a pixel at (x,y)
	 virtual void GetRGBA(Sint16 x, Sint16 y, Uint8 * r, Uint8 * g, Uint8 * b, Uint8 * a) = 0;
	 /// return Alpha value of a pixel at (x,y)
	 virtual Uint8 GetAlpha(Sint16 x, Sint16 y) = 0;

	 /// return true if the point p is inside the surface
	 virtual bool IsInside(Point & p) { return IsInside(p.x, p.y); };
	 /// return true if the point (x,y) is inside the surface
	 virtual bool IsInside(Sint16 x, Sint16 y)
	 {
		 if(x >= _rectangle->x && x <= (_rectangle->x + _rectangle->w) &&
			y >= _rectangle->y && y <= (_rectangle->y + _rectangle->h))
			 return true;
		 else
			 return false;
	 };

	 /// lock the surface
	 virtual void Lock() = 0;
	 /// relase the previous lock
	 virtual void Unlock() = 0;

	 /// update texture
	 virtual void UpdateTex(bool del_old_tex = true) = 0;
};

}

#endif
